//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : cursor.h  
//
// Classes     : Cursors
//
// Description : a class to hold cursor data for each input window,  
//               positions both in the window and in the data.             
//
// Copyright   : (c) 1991, 1993 Ian Craggs               
//
//////////////////////////////////////////////////////////////////////////////

#if !defined(CURSOR_H)
#define CURSOR_H

#include "Globals.h"
#include "Line.h"

class Cursors
{
public:

  Cursors()
    { fDisplayRowCol = TRUE; Reset(NULL); }

  void Reset(Lines *);

  void Move(direction);

  void Show();

  int CurColOf()
    { return theCurCol; }

  int CurRowOf()
    { return theCurRow; }

  int RowOf()
    { return theRow; }

  int ColOf()
    { return theCol; }

  char *DataOf()
    { return theCurLine->DataOf(); }

  char *EOL()
    { return theCurLine->DataOf(theCurCol + 1); }      

  Lines *CurLineOf()
    { return theCurLine; }

  void setCurLine(Lines *aLine)
    { theCurLine = aLine; }

  void setRow(int aRow)
    { theRow = aRow; }

  void setCol(int aCol)
    { theCol = aCol; }

  void setCurRow(int aRow)
    { theCurRow = aRow; }

  void setCurCol(int aCol)
    { theCurCol = aCol; }

  void Home();

  bool fDisplayRowCol;                 // Display row/col on status line?

private:

  int   theRow,                        // Cursor row in window
	theCol;                        // Cursor col in window

  int   theCurCol;                     // Cursor col in data
  int   theCurRow;                     // Cursor row in data
  Lines *theCurLine;                   // Cursor line in data

}; // end class cursor

#endif
