//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : cwindow.h
//
// Classes     : CommandWindows
//
// Description : derived from InputWindows, runs commands.                     
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#include "Cwindow.h"
#include "Keyboard.h"
#include "Mwindow.h"

#include <stdlib.h>
#include <string.h>
#include <ctype.h>


CommandWindows::CommandWindows()
{
  theNoOfCommands = 10;

  theBaseCol = 1;
  theBaseRow = new Lines("      ");
  Lines *theCurLine = theBaseRow;

  for (int i = 0; i < theNoOfCommands; ++i)
  {
    Lines *theNewLine = new Lines("      ");
    theCurLine->SetNext(theNewLine);
    theNewLine->SetPrev(theCurLine);
    theCurLine = theNewLine;
  } // endfor

  theCurLine->SetNext(theBaseRow);
  theBaseRow->SetPrev(theCurLine);

  theLastRow = theBaseRow;

  theCursor.Reset(theBaseRow);
  theCursor.fDisplayRowCol = FALSE;
} // end CommandWindows constructor


CommandWindows::~CommandWindows()
{
  Lines *theCurLine = theBaseRow;

  for (int i = 0; i < theNoOfCommands; ++i)
  {
    Lines *NextLine = theCurLine->NextOf();
    delete theCurLine;
    theCurLine = NextLine;
  } // endfor

} // end CommandWindows destructor


static struct CmdString
{
  char* string;
  OPCODE op;
} CmdTable[] =
{
  {"QUIT", QUIT},
  {"Q",    QUIT},
  {"SAVE", SAVE},
  {"S",    SAVE},
  {"EDIT", EDIT},
  {"E",    EDIT},
  {"N",    NAME},
  {"NAME", NAME},
  {"DOS",  DOS},
  {"GOTO", GOTO},
  {"",     NOP}
};


void CommandWindows::Enter()
{
  Lines ComLine;
  ComLine.SetData(theBaseRow->DataOf());

  char *pCh = ComLine.DataOf();
  while (*pCh)                         // Skip preceding blanks.
  {
    if (*pCh != ' ')
      break;
    ++pCh;
  } // endwhile

  if (atoi(pCh) != 0)
  {
    theKeyboard.InsertOp(GOTO);
    theKeyboard.InsertParm(pCh);
    CursorDown();
    CursorHome();
    return;
  } // endif

  char ch = toupper(*pCh);

  if (ch == '.' || ch == '/' || ch == 'L' ||  ch == 'C')
  {
    if (ch == '.')
      theKeyboard.InsertOp(DOS);
    else if (ch == 'C')
      theKeyboard.InsertOp(CHANGE);
    else
      theKeyboard.InsertOp(FIND);
    if (ch != '/')                     // Pass the '/' too!
      ++pCh;
    theKeyboard.InsertParm(pCh);
    CursorDown();
    CursorHome();
    return;
  } // endif

  char* cmd = strupr(strtok(ComLine.DataOf(), " "));

  struct CmdString* CurString = &CmdTable[0];

  while (CurString->string[0] != '\0')
  {
    if (strcmp(cmd, CurString->string) == 0)
    {
      theKeyboard.InsertOp(CurString->op);
      cmd = strtok(NULL, " ");
      theKeyboard.InsertParm(cmd);
      CursorDown();
      CursorHome();
      return;
    } // endif
    ++CurString;
  } // endwhile

  theMsgWind.setMsg("Unknown command.");

} // end Enter


RETCODE CommandWindows::Refresh()
{
  theCursor.setCurLine(theBaseRow);
  RefreshLine();
  return OK;
} // end Refresh



