//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : ebuf.h
//
// Classes     : EditBuffers
//
// Description : a class to hold a line of data being edited for the
//               InputWindows class.                                       
//
// Copyright   : (c) 1991, 1993 Ian Craggs               
//
//////////////////////////////////////////////////////////////////////////////

#include <conio.h>

#include "ebuf.h"


RETCODE EditBuffers::EraseEOL()
{
  RETCODE rc = OK;
  if (CanEdit())
    theCurLine->EraseEOL(theCursor->CurColOf());
  else
    rc = CANT_EDIT;
  return rc;
} // end EditBuffers::EraseEOL


RETCODE EditBuffers::ReplaceChar(char aChar)
{
  RETCODE rc = OK;
  if (CanEdit())
    theCurLine->ReplaceChar(aChar, theCursor->CurColOf());
  else
    rc = CANT_EDIT;
  return rc;
} // end EditBuffers::ReplaceChar


RETCODE EditBuffers::InsertChar(char aChar)
{
  RETCODE rc = OK;
  if (CanEdit())
    theCurLine->InsertChar(aChar, theCursor->CurColOf());
  else
    rc = CANT_EDIT;
  return rc;
} // end EditBuffers::InsertChar


RETCODE EditBuffers::DeleteChar()
{
  RETCODE rc = OK;
  if (CanEdit())
    theCurLine->DeleteChar(theCursor->CurColOf());
  else
    rc = CANT_EDIT;
  return rc;
} // end EditBuffers::DeleteChar


void EditBuffers::Rollback()
{
  if (!isStillEditing())
    return;

  theCurLine->Replace(theOriginalLine);
  isEditing = FALSE;
  --theNoOfChanges;
} // end EditBuffers::Rollback


EditBuffers::EditBuffers()
{
  theNoOfChanges = 0;
  theOriginalLine = new Lines;
  theCurLine = NULL;
  isEditing = FALSE;
} // end EditBuffers::EditBuffers


EditBuffers::~EditBuffers()
{
  Commit();
  delete theOriginalLine;
} // end EditBuffers::~EditBuffers


bool EditBuffers::isStillEditing()
{
  if (isEditing == TRUE && theCurLine == theCursor->CurLineOf())
    return TRUE;
  else
    return FALSE;
} // end isStillEditing


bool EditBuffers::CanEdit()
{
  return (isStillEditing() || EditNewLine()) ? TRUE : FALSE;
} // end CanEdit


bool EditBuffers::EditNewLine()
{
  Commit();                            // Save any previously unsaved line.

  if ((theCurLine = theCursor->CurLineOf()) == NULL)
    return FALSE;

  if (theCurLine->isReadOnly)
    return FALSE;

  if (theOriginalLine->SetData(theCurLine->DataOf()) == OK)
  {
    isEditing = TRUE;
    ++theNoOfChanges;
  } // endif

  return isEditing;
} // end EditBuffers::EditNewLine


void EditBuffers::Commit()
{
  if (!isEditing)
    return;

  theCurLine->Strip();                 // Strip blanks & insert tabs.

  isEditing = FALSE;

  return;
} // end EditBuffers::Commit




