//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : ebuf.h
//
// Classes     : EditBuffers
//
// Description : a class to hold a line of data being edited for the
//               InputWindows class.                                       
//
// Copyright   : (c) 1991, 1993 Ian Craggs               
//
//////////////////////////////////////////////////////////////////////////////

#if !defined(EBUF_H)
#define EBUF_H

#include "globals.h"
#include "cursor.h"


class EditBuffers          
{
public:
          
  RETCODE EraseEOL();                  // Erase to end of line.  

  RETCODE ReplaceChar(char);           // Replace a character.

  RETCODE InsertChar(char);            // Insert a character.

  RETCODE DeleteChar();                // Delete a character.

  void Rollback();       	       // Restore old line to file.

  void SetCursor(Cursors *aCursor)
    { theCursor = aCursor; }

  EditBuffers();                       // Constructor.

  ~EditBuffers();                      // Destructor.

  int NoOfChanges()
    { return theNoOfChanges; }

  void resetChanges()
    { theNoOfChanges = 0; }

  void IncrChanges()
    { ++theNoOfChanges; }              // To allow Marks to add changes.

private:

  bool   isStillEditing();             // Cursor still on same line?

  bool   CanEdit();                    // Can edit this line?

  bool   EditNewLine();                // Edit a new line.

  void   Commit();                     // Commit changed line to file.

  bool   isEditing;                    // Is a line being edited?

  int    theNoOfChanges;               // No. of changes since last save.

  Lines  *theOriginalLine;             // The original data.

  Lines  *theCurLine;                  // The current line.

  Cursors *theCursor;                  // The current cursor.

}; // end class ebuf

#endif
