//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : editor.cpp         
//
// Classes     : Editors
//
// Description : the main class of the editor, Run() is the main loop.     
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#include "Editor.h"

#define KEYBOARD_OWNER
#define SCREEN_OWNER
#define MSG_WIND_OWNER

#include "Keyboard.h"
#include "Screen.h"
#include "Mwindow.h"
#include "Mark.h"

#include <ctype.h>
#include <stdlib.h>


static char* HelpLine = "F1=Help F2=Save F3=Quit F4=File F5=Dos  "
			"        F7=Name F8=Edit F9=Undo F10=Next";


void Editors::Run(char *aPath)
{
  OPCODE op;

// Set up message line
  theMsgWind.Setup(1, theScreen.RowsOf() - 1, theScreen.ColsOf(),
		      theScreen.RowsOf() - 1, LIGHTRED, BLACK);
  theMsgWind.setMsg("Loading...");

// Set up help/function line
  theHelpWind.Setup(1, theScreen.RowsOf(), theScreen.ColsOf(),
		       theScreen.RowsOf(), YELLOW, BLACK);
  theHelpWind.DisplayLine(HelpLine);

  theFirstFile = theCurFile =
	   new FileWindows(aPath);     // Load the first file.

  theMsgWind.setMode(theKeyboard.isInInsertMode());

  theMsgWind.setMsg("Ian's personal editor    ...          (c)"
		    " 1991, 1993 Ian Craggs");

// Set up command line
  theCmdLine.Setup(1, theScreen.RowsOf() - 2, theScreen.ColsOf(),
		      theScreen.RowsOf() - 2, BLACK, CYAN);
  if (isOnCmdLine)
  {
    theCurWindow = &theCmdLine;
    theCmdLine.Refresh();
  }
  else
    theCurWindow = theCurFile;


  while ((op = theKeyboard.getOp()) != TERMINATE)
  {
    if (op == NOP)
      continue;

    if (theMsgWind.isMessageActive())
      theMsgWind.resetMsg();

    switch (op)
    {
      case ASCII :
	theCurWindow->Type(theKeyboard.getChar());
	break;

      case PGDN :
	theCurFile->PgDn();
	if (isOnCmdLine)
	  theCurWindow->Refresh();
	break;

      case PGUP :
	theCurFile->PgUp();
	if (isOnCmdLine)
	  theCurWindow->Refresh();
	break;

      case CUR_LEFT :
	theCurWindow->CursorLeft();
	break;

      case CUR_RIGHT :
        theCurWindow->CursorRight();
        break;

      case CUR_DOWN :
        theCurWindow->CursorDown();
        break;

      case CUR_UP :
        theCurWindow->CursorUp();
	break;

      case HOME:
	theCurWindow->CursorHome();
	break;

      case END:
	theCurWindow->CursorEnd();
	break;

      case DELETE :
	theCurWindow->DeleteChar();
	break;

      case BACKSP :
	theCurWindow->BackSpace();
	break;

      case ERASE_EOL :
	theCurWindow->EraseEOL(); 
	break;

      case UNDO :
	theCurWindow->Rollback();
	break;

      case SAVE :
        theCurFile->Save();
	break;

      case ENTER :
        theCurWindow->Enter();
	break;

      case TAB :
	theCurWindow->Tab();
	break;

      case BACK_TAB :
	theCurWindow->BackTab();
	break;

      case DELETE_LINE :
	if (!isOnCmdLine)
	  theCurFile->DeleteLine();
	break;

      case COMMAND :
	isOnCmdLine = (isOnCmdLine) ? FALSE : TRUE;
	if (isOnCmdLine)
	  theCurWindow = &theCmdLine;
	else
	  theCurWindow = theCurFile;
	theCurWindow->Refresh();
	break;

      case COMMAND_TYPE :
	if (!isOnCmdLine)
	{
	  theCurWindow = &theCmdLine;
	  theCurWindow->Refresh();
	} // endif
	char* pCh = theKeyboard.ParmOf();
	while (*pCh != '\0')
	  theCurWindow->Type(*pCh++);
	break;

      case COMMAND_EDIT:
	theKeyboard.InsertOp(COMMAND_TYPE);
	theKeyboard.InsertParm("EDIT ");
	break;

      case COMMAND_NAME:
	theKeyboard.InsertOp(COMMAND_TYPE);
	theKeyboard.InsertParm("NAME ");
	break;

      case INSERT :
	theKeyboard.ToggleInsert();
	break;

      case MARK_LINE :
	if (!isOnCmdLine)
	  theCurFile->MarkLine(theMark);
	break;

      case UNMARK :
	bool isCurrent =
	  (theCurFile == theMark.FileWindowOf()) ? TRUE : FALSE;
	theMark.Clear();
	if (isCurrent)
	  theCurFile->Refresh();
	break;

      case SHIFT_MARK_RIGHT :
	isCurrent =
	  (theCurFile == theMark.FileWindowOf()) ? TRUE : FALSE;
	theMark.Shift(RIGHT);
	if (isCurrent)
	  theCurFile->Refresh();
	break;

      case SHIFT_MARK_LEFT :
	isCurrent =
	  (theCurFile == theMark.FileWindowOf()) ? TRUE : FALSE;
	theMark.Shift(LEFT);
	if (isCurrent)
	  theCurFile->Refresh();
	break;

      case DELETE_MARK :
	isCurrent =
	  (theCurFile == theMark.FileWindowOf()) ? TRUE : FALSE;
	theMark.Delete();
	if (isCurrent)
	  theCurFile->Refresh();
	break;

      case COPY_MARK :
	if (!isOnCmdLine)
	{
	  theMark.Copy(*theCurFile);
	  theCurFile->Refresh();
	} // endif
	break;

      case MOVE_MARK :
	if (!isOnCmdLine)
	{
	  theMark.Move(*theCurFile);
	  theCurFile->Refresh();
	} // endif
	break;

      case EDIT :
	FileWindows* NewOne = new FileWindows(theKeyboard.ParmOf());
	if (NewOne != NULL)
	{
	  theCurFile->setNext(NewOne);
	  (theCurFile->NextOf())->setPrev(theCurFile);
	  theCurFile = theCurFile->NextOf();
	  if (!isOnCmdLine)
	    theCurWindow = theCurFile;
	  else
	    theCurWindow->Refresh();
	} // endif
	break;

      case NEXT_FILE :
	if (theCurFile->NextOf() == NULL)
	  theCurFile = theFirstFile;
	else
	  theCurFile = theCurFile->NextOf();
	theCurFile->Refresh();
	if (!isOnCmdLine)
	  theCurWindow = theCurFile;
	else
	  theCurWindow->Refresh();
	break;

      case PREV_FILE :
	if (theCurFile->PrevOf() == NULL)
	  while (theCurFile->NextOf() != NULL)
	    theCurFile = theCurFile->NextOf();
	else
	  theCurFile = theCurFile->PrevOf();
	theCurFile->Refresh();
	if (!isOnCmdLine)
	  theCurWindow = theCurFile;
	else
	  theCurWindow->Refresh();
	break;

      case SAVEQUIT :
	if (theCurFile->NoOfChanges() > 0)
	  theCurFile->Save();

	// Drop into the QUIT section i.e. no break
	// i.e. DON'T PUT A BREAK HERE!!!

      case QUIT :
	char ch = 'Y';
	if (theCurFile->NoOfChanges() > 0)
	{
	  theMsgWind.setMsg("Throw away changes? (y/n)");

	  for (;;)
	  {
	    OPCODE opCode = theKeyboard.getOp();
	    if (opCode == ASCII)
	    {
	      ch = toupper(theKeyboard.getChar());
	      if (ch == 'N' || ch == 'Y')
		break;
	    } // endif
	  } // endfor
	  theMsgWind.resetMsg();
	} // endif
	if (ch == 'N')
	  break;

	if (theCurFile == theFirstFile
	 && theCurFile->NextOf() == NULL)
	  theKeyboard.InsertOp(TERMINATE);
	else
	{
	  FileWindows* theFileToBeDeleted = theCurFile;
	  FileWindows* theNextFile = theCurFile->NextOf();
	  FileWindows* thePrevFile = theCurFile->PrevOf();
	  if (theNextFile != NULL)
	  {
	    theNextFile->setPrev(thePrevFile);
	    theCurFile = theNextFile;
	  } // endif

	  if (thePrevFile != NULL)
	  {
	    thePrevFile->setNext(theNextFile);
	    theCurFile = thePrevFile;
	  }
	  else
	    theFirstFile = theCurFile;
          
	  if (theFileToBeDeleted == theMark.FileWindowOf())
	    theMark.Clear();      
          
	  delete theFileToBeDeleted;
	  theCurFile->Refresh();
	  if (!isOnCmdLine)
	    theCurWindow = theCurFile;
	  else
	    theCurWindow->Refresh();
	} // endif
	break;

      case DOS :
	theScreen.Clear();
	system(theKeyboard.ParmOf());  // Execute command.
	if (theKeyboard.ParmOf()[0] != '\0')
	{
	  PutString("Press any key to continue...", 28);
	  theKeyboard.getOp();
	} // endif
	theHelpWind.DisplayLine(HelpLine);
	theCurFile->Refresh();
	if (isOnCmdLine)
	  theCurWindow->Refresh();
	break;

      case NAME :                      // Rename current file.
	theCurFile->Rename(theKeyboard.ParmOf());
	break;

      case GOTO :
	theCurFile->Goto(atoi(theKeyboard.ParmOf()));
	theCurWindow->Refresh();       // Must be on command line.
	break;

      case FIND :
	if (theCurFile->Find(theKeyboard.ParmOf()) == OK
	 && isOnCmdLine)
	{
	  isOnCmdLine = FALSE;
	  theCurWindow = theCurFile;
	} // endif
	break;

      case FIND_AGAIN :
	if (theCurFile->Find(theKeyboard.ParmOf(), TRUE) == OK
	 && isOnCmdLine)
	{
	  isOnCmdLine = FALSE;
	  theCurWindow = theCurFile;
	} // endif
	break;

      case CHANGE :
	if (theCurFile->Change(theKeyboard.ParmOf()) == OK
	 && isOnCmdLine)
	{
	  isOnCmdLine = FALSE;
	  theCurWindow = theCurFile;
	} // endif
	break;

      case TOP_OF_FILE :
	theCurFile->Top();
	break;

      case BOTTOM_OF_FILE :
	theCurFile->Bottom();
	break;

      case SPLIT :
	if (!isOnCmdLine)
	  theCurFile->SplitLine();
	break;

      case JOIN :
	if (!isOnCmdLine)
	  theCurFile->JoinLines();
	break;

      case HELP:
	theKeyboard.InsertOp(EDIT);
	theKeyboard.InsertParm("\\I.HLP");
	break;

    } // endswitch

  } // endwhile

} // end Editors::Run





