//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : file.cpp
//
// Classes     : Files
//
// Description : the File class holds the data for one file.  It is used in
//               the FileWindows class to provide a window onto the data in
//               the file.  It uses the Lines class to hold each line of
//               data in the file in a doubly-linked list.       
//
// Copyright   : (c) 1991, 1993 Ian Craggs               
//
//////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>

#include "file.h"
#include "cursor.h"


static char *StartLine = " Top of File ";
static char *EndLine   = " End of File ";


Files::Files()
{              
  thePath = NULL;                      // Path of this file.
  theFirstLine = NULL;                 // First line of file.
} // end constructor


Files::~Files()
{
  delete thePath;
  Lines *theCurLine = theFirstLine;

  while (theCurLine != NULL)
  {
    Lines *theNextLine = theCurLine->NextOf();
    delete theCurLine;
    theCurLine = theNextLine;
  } // endwhile
} // end Files destructor


RETCODE Files::Rename(char *aPath)
{
  if (strlen(aPath) > strlen(thePath))
  {
    delete[] thePath;        
    if ((thePath = new char[strlen(aPath) + 1]) == NULL)
      return NO_MORE_MEMORY;
  } // endif
  strcpy(thePath, aPath);
  return OK;
} // end Rename


RETCODE Files::Load(char *aPath, Lines **FirstLineToSet)
{
  FILE  *InFile;
  Lines *CurLine,
	*OldLine = NULL;
  RETCODE rc;

  theNoOfLines = 0;

  *FirstLineToSet = NULL;

  if ((thePath = new char[strlen(aPath) + 1]) == NULL)
    return NO_MORE_MEMORY;

  strcpy(thePath, aPath);

//*************************************************************************
// Set first line.
//*************************************************************************
  if ((OldLine = new Lines) == NULL)
    return NO_MORE_MEMORY;

  if ((rc = OldLine->SetData(StartLine)) != OK)
    return rc;

  theFirstLine = OldLine;
  theFirstLine->isReadOnly = TRUE;

//*************************************************************************
// Read the rest of the lines.
//*************************************************************************
  if ((InFile = fopen(aPath, "rt")) == NULL)         
  {
    if ((CurLine = new Lines) == NULL)
      return NO_MORE_MEMORY;
    if ((rc = CurLine->SetData("     ")) != OK)
      return rc;
    OldLine->SetNext(CurLine);
    CurLine->SetPrev(OldLine);
    OldLine = CurLine;
  }
  else
  {
    char Buffer[MAX_LINE_SIZE];
    while (fgets(Buffer, MAX_LINE_SIZE, InFile) != NULL)
    {
      if ((CurLine = new Lines) == NULL)
	return NO_MORE_MEMORY;
      if ((rc = CurLine->SetData(Buffer)) != OK)
	return rc;

      ++theNoOfLines;
      OldLine->SetNext(CurLine);
      CurLine->SetPrev(OldLine);
      OldLine = CurLine;

    } // endwhile
    fclose(InFile);
  } // endif

//*************************************************************************
// Set last line
//*************************************************************************
  if ((CurLine = new Lines) == NULL)
    return NO_MORE_MEMORY;
  if ((rc = CurLine->SetData(EndLine)) != OK)
    return rc;
  OldLine->SetNext(CurLine);
  CurLine->SetPrev(OldLine);
  theLastLine = CurLine;
  theLastLine->isReadOnly = TRUE;

  *FirstLineToSet = theFirstLine;
  return rc;
} // end Load



RETCODE Files::Save(void)
{
  FILE  *OutFile;
  Lines *CurLine;

  if ((OutFile = fopen(thePath, "w")) == NULL)
    return FILE_NOT_OPENED;

  CurLine = theFirstLine->NextOf();
  while (!isFarDown(CurLine))
  {
    fprintf(OutFile, "%s\n", CurLine->DataOf());
    CurLine = CurLine->NextOf();
  } // endwhile

  fclose(OutFile);

  return OK;
} // end Save


RETCODE Files::DeleteLine(Cursors *aCursor)
{
  Lines *CurLine = aCursor->CurLineOf();

  if (CurLine->isReadOnly)
    return CANT_EDIT;

  Lines *NextLine = CurLine->NextOf();
  Lines *PrevLine = CurLine->PrevOf();

  NextLine->SetPrev(PrevLine);
  PrevLine->SetNext(NextLine);

  delete CurLine;

  aCursor->setCurLine(NextLine);

  --theNoOfLines;

  return OK;
} // end DeleteLine


RETCODE Files::InsertLine(Cursors *aCursor)
{
  static char NewBuf[11] = "          ";
  Lines *NewLine,
	*NextLine,
	*CurLine = aCursor->CurLineOf();

  if (CurLine->NextOf() == NULL)
    return CANT_EDIT;
          
  if ((NewLine = new Lines) == NULL)
    return NO_MORE_MEMORY;

  NewLine->SetData(NewBuf);

  NextLine = CurLine->NextOf();

  NewLine->SetNext(CurLine->NextOf());
  NewLine->SetPrev(NextLine->PrevOf());

  CurLine->SetNext(NewLine);
  NextLine->SetPrev(NewLine);

  ++theNoOfLines;

  return OK;
} // end Files::InsertLine

