//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : file.h
//
// Classes     : Files
//
// Description : the File class holds the data for one file.  It is used in
//               the FileWindows class to provide a window onto the data in
//               the file.  It uses the Lines class to hold each line of
//               data in the file in a doubly-linked list.       
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#if !defined(FILES_H)
#define FILES_H

#include "Globals.h"
#include "Line.h"

const int MAX_LINE_SIZE = 512;

class Cursors;

class Files
{
public:

  Files();                             // Constructor.

  ~Files();                            // Destructor.

  RETCODE Load(char *, Lines **);      // Load file from disk.

  RETCODE Rename(char *);              // Change the file's name.

  RETCODE Save(void);                  // Save file to disk.

  RETCODE InsertLine(Cursors *);       // Insert blank line at cursor position.

  RETCODE DeleteLine(Cursors *);       // Delete line at cursor position.

  char *NameOf()                       // The name of the file.
    { return thePath; }

  bool isFarUp(Lines *aLine)           // Is the line at the top of the file?
    { return (aLine == theFirstLine) ? TRUE : FALSE; }

  bool isFarDown(Lines *aLine)         // Is the line at the bottom?
    { return (aLine == theLastLine) ? TRUE : FALSE; }

private:

  char  *thePath;                      // Path of this file.
  Lines *theFirstLine;                 // First line of file.
  Lines *theLastLine;                  // Last line of file.
  int   theNoOfLines;                  // Number of lines in file.

}; // end class Files

#endif
