#include "Fwindow.h"
#include "Screen.h"
#include "Keyboard.h"
#include "Mwindow.h"
#include "Mark.h"

#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>


static char* StrStr(char*, char*, bool);


FileWindows::FileWindows(char *aPath)
{
  isFirstLocate = TRUE;
  thePrev = theNext = NULL;
  Setup(1, 1, theScreen.ColsOf(),
	      theScreen.RowsOf() - 2, GREEN, BLACK);
  if (theFile.Load(aPath, &theBaseRow) == NO_MORE_MEMORY)
  {
    theMsgWind.setMsg("Insufficient memory to load file.");
    exit(NO_MORE_MEMORY);
  } // endif
  theCursor.Reset(theBaseRow);         // Reset cursor position.
  theBaseCol = 1;
  CursorDown();                        // Move cursor to first line.
  Refresh();
} // end FileWindows::Load


void FileWindows::Top()
{
  Lines *CurLine = theBaseRow;

  int theRowNo = theCursor.CurRowOf();

  if (theFile.isFarUp(CurLine))
    return;

  while (!theFile.isFarUp(CurLine))
  {
    CurLine = CurLine->PrevOf();
    --theRowNo;
  } // endwhile

  theCursor.setCurRow(theRowNo);

  theBaseRow = CurLine;
  Refresh();
} // end Top


void FileWindows::Bottom()
{
  Lines* CurLine = theCursor.CurLineOf();

  while (theLastRow->NextOf() != NULL)
  {
    CurLine = CurLine->NextOf();
    theBaseRow = theBaseRow->NextOf();
    theLastRow = theLastRow->NextOf();

    theCursor.setCurRow(theCursor.CurRowOf() + 1);
    theCursor.setCurLine(theCursor.CurLineOf()->NextOf());
  } // endwhile

  Refresh();
} // end Bottom


void FileWindows::Goto(int LineNo)
{
  if (LineNo == 0)
    return;

  while (LineNo > theCursor.CurRowOf() &&
	 theCursor.CurLineOf()->NextOf() != NULL)
  {
    if (theLastRow->NextOf() != NULL)
    {
      theBaseRow = theBaseRow->NextOf();
      theLastRow = theLastRow->NextOf();

      theCursor.setCurRow(theCursor.CurRowOf() + 1);
      theCursor.setCurLine(theCursor.CurLineOf()->NextOf());
    }
    else
      theCursor.Move(DOWN);
  } // endwhile

  while (LineNo < theCursor.CurRowOf() &&
	 theCursor.CurLineOf()->PrevOf() != NULL)
  {
    if (theBaseRow->PrevOf() != NULL)
    {
      theBaseRow = theBaseRow->PrevOf();
      theLastRow = theLastRow->PrevOf();

      theCursor.setCurRow(theCursor.CurRowOf() - 1);
      theCursor.setCurLine(theCursor.CurLineOf()->PrevOf());
    }
    else
      theCursor.Move(UP);
  } // endwhile

  Refresh();

} // end Goto


RETCODE FileWindows::Refresh()
{
  int i = 1;
  Lines *CurLine = theBaseRow;

  _setcursortype(_NOCURSOR);

  while (CurLine != NULL && i <= theNoOfRows)
  {
    DisplayLine(CurLine, i++, theBaseCol);
    theLastRow = CurLine;
    CurLine = CurLine->NextOf();
  } // endwhile

  while (i <= theNoOfRows)             // Clear the rest of the rows.
  {
    gotoxy(1, i++);
    clreol();
  }// endwhile

  theKeyboard.setCursor();
  ResetCursor();

  theMsgWind.setFileName(theFile.NameOf());

  return OK;

} // end Refresh


void FileWindows::PgDn()
{
  int i = 1;
  Lines *CurLine = theBaseRow;

  int theRowNo = theCursor.CurRowOf();

  if (theFile.isFarDown(theLastRow))
    return;

  while (i++ <= theNoOfRows && !theFile.isFarDown(theLastRow))
  {
    CurLine = CurLine->NextOf();
    theLastRow = theLastRow->NextOf();
    ++theRowNo;
  } // endwhile

  theBaseRow = CurLine;
  theCursor.setCurRow(theRowNo);

  if (theFile.isFarDown(CurLine))
    RowsUp(theNoOfRows - 1);
  else
    Refresh();

} // end PgDn


void FileWindows::RowsUp(int aNoOfRows)
{
  int i = 1;
  Lines *CurLine = theBaseRow;

  int theRowNo = theCursor.CurRowOf();

  if (theFile.isFarUp(CurLine))
    return;

  while (i++ <= aNoOfRows && !theFile.isFarUp(CurLine))
  {
    CurLine = CurLine->PrevOf();
    --theRowNo;
  } // endwhile

  theCursor.setCurRow(theRowNo);

  theBaseRow = CurLine;
  Refresh();
} // end RowsUp


void FileWindows::PgUp()
{
  RowsUp(theNoOfRows);
} // end pgup


void FileWindows::ResetCursor()
{
  int i = 1;
  Lines *CurLine = theBaseRow;

  while (i < theCursor.RowOf())
  {
    if (theFile.isFarDown(CurLine))
    {
      theCursor.setRow(i);
      break;
    } // endif
    CurLine = CurLine->NextOf();
    ++i;
  } // endwhile

  theCursor.setCurLine(CurLine);
  theCursor.Show();
} // end FileWindows::ResetCursor


void FileWindows::Enter()
{
  RETCODE rc = OK;

  if (theKeyboard.isInInsertMode())
  {
    rc = theFile.InsertLine(&theCursor);
    Refresh();
  } // endif

  if (rc == OK)
  {
    CursorDown();
    CursorHome();
  } // endif
} // end FileWindows::InsertLine


void FileWindows::Rename(char* aPath)
{
  theFile.Rename(aPath);
  theMsgWind.setFileName(theFile.NameOf());
} // end Rename


void FileWindows::MarkLine(Marks& aMark)
{
  aMark.Set(theCursor, *this);
  Refresh();
} // end MarkLine


RETCODE FileWindows::Locate(char* aString, bool isCaseSensitive)
{
  Lines* CurLine = theCursor.CurLineOf();
  int    CurCol  = theCursor.CurColOf() + 2;  // Start after cursor position.
  char*  Data    = CurLine->DataOf(CurCol);
  char*  pFound  = NULL;

  while (!theFile.isFarDown(CurLine))
  {
    pFound = StrStr(Data, aString, isCaseSensitive);
    if (pFound != NULL)
      break;
    CurCol = 0;
    CurLine = CurLine->NextOf();
    Data    = CurLine->DataOf();
  } // endwhile

  if (pFound == NULL)
  {
    theMsgWind.setMsg("Not found.");
    return CANT_FIND;
  }
  else
  {
    // position file & cursor.
    while (theCursor.CurLineOf() != CurLine)
    {
      theBaseRow = theBaseRow->NextOf();
      theLastRow = theLastRow->NextOf();

      theCursor.setCurRow(theCursor.CurRowOf() + 1);
      theCursor.setCurLine(theCursor.CurLineOf()->NextOf());

    } // endwhile
    MoveCursorColTo(pFound - theCursor.CurLineOf()->DataOf());
    Refresh();
    return OK;
  } // endif
} // end Locate


RETCODE FileWindows::Find(char* CmdString, bool Again)
{
  if (Again)
  {
    if (isFirstLocate)
    {
      theMsgWind.setMsg("No previous find executed.");
      return CANT_FIND;
    }
    else
      return Locate(FirstParm.DataOf(), isCaseSensitive);
  } // endif

  isCaseSensitive = FALSE;
  isFirstLocate   = FALSE;

  char* pCh = CmdString;               // Point to command string.

  while (*pCh && *pCh != ' ')          // Skip all preceding blanks.
    break;

  if (*pCh == '\0')                    // Nothing to do.
    return OK;

  char  delimiter  = *pCh;             // Parm delimiter.
  char* pFirstParm = ++pCh;            // Pointer to first parm.

  while (*pCh && *pCh != delimiter)    // Look for end of parm.
    ++pCh;

  if (*pCh == '\0')                    // If no terminating delimiter,
    while (*(--pCh) == ' ')            // go back to last non-blank char.
      ;

  if (*pCh == delimiter)
  {
    *(pCh++) = '\0';

    while (*pCh && *pCh != ' ')        // Skip all preceding blanks.
      break;
    if (toupper(*pCh) == 'S')
      isCaseSensitive = TRUE;
  }
  else
    *(++pCh) = '\0';                   // Terminate first parm.

  FirstParm.SetData(pFirstParm);

  return Locate(pFirstParm, isCaseSensitive);

} // end Find


RETCODE FileWindows::Change(char* CmdString)
{
  CHANGE_OPTS ChangeOptions = NONE;
  char *Buffer = new char[strlen(CmdString) + 1];
  strcpy(Buffer, CmdString);
  char* pCh = Buffer;                  // Point to copy of command string.

  while (*pCh && *pCh != ' ')          // Skip all preceding blanks.
    break;

  if (*pCh == '\0')                    // Nothing to do.
    return OK;

  char  delimiter  = *pCh;             // Parm delimiter.
  char* pFirstParm = ++pCh;            // Pointer to first parm.

  while (*pCh && *pCh != delimiter)    // Look for end of parm.
    ++pCh;

  if (*pCh == '\0')                    // If no terminating delimiter,
  {                                    // error.
    theMsgWind.setMsg("No string to change to.");
    return CANT_FIND;
  } // end if

  *pCh = '\0';                         // Terminate first parm.

  char* pSecondParm = ++pCh;           // Pointer to second parm.
  while (*pCh && *pCh != delimiter)    // Look for end of parm.
    ++pCh;

  if (*pCh == '\0')                    // If no terminating delimiter,
    while (*(--pCh) == ' ')            // go back to last non-blank char.
      ;

  if (*pCh == delimiter)               // If closing delimiter is found,
  {                                    // look for options.
    *(pCh++) = '\0';

    while (*pCh && *pCh != ' ')        // Skip all preceding blanks.
      break;

    switch (toupper(*pCh))
    {
    case 'M':
      ChangeOptions = MARKED_LINES_ONLY;
      break;

    case '*':
      ChangeOptions = ALL_OCCURRENCES;
      break;

    } // end switch
  }
  else
    *(++pCh) = '\0';                   // Terminate second parm.

  FirstParm.SetData(pFirstParm);

  RETCODE rc = ExecuteChange(pFirstParm, pSecondParm, ChangeOptions);

  delete Buffer;

  return rc;

} // end Change


RETCODE FileWindows::ExecuteChange(char *String1,
				   char *String2, CHANGE_OPTS opts)
{
  int Str1Len = strlen(String1);
  bool inquire = (opts == NONE) ? TRUE : FALSE;
  RETCODE rc;

  if ((rc = Locate(String1, TRUE)) != OK)
    return rc;

  while (rc == OK)
  {
    Lines* pCurLine = theCursor.CurLineOf();
    if (inquire)
    {
      char ch;
      theMsgWind.setMsg("Yes/No/Go/Last/Quit ?");

      for (;;)
      {
	OPCODE opCode = theKeyboard.getOp();
	if (opCode == ASCII)
	{
	  ch = toupper(theKeyboard.getChar());
	  if (ch == 'Y' || ch == 'N' || ch == 'G' || ch == 'L' || ch == 'Q')
	    break;
	} // endif
      } // endfor
      theMsgWind.resetMsg();
      switch (ch)
      {
      case 'G':
	inquire = FALSE;
      case 'Y':
	pCurLine->ReplaceString(theCursor.CurColOf(), Str1Len, String2);
	theEditBuffer.IncrChanges();
	break;

      case 'N':                        // Do nothing here.
	break;

      case 'L':
	pCurLine->ReplaceString(theCursor.CurColOf(), Str1Len, String2);
	theEditBuffer.IncrChanges();
      case 'Q':
	goto finish;                   // No more to do in this function.
      } // end switch
    }
    else
    {
      pCurLine->ReplaceString(theCursor.CurColOf(), Str1Len, String2);
      theEditBuffer.IncrChanges();
    } // end if
    rc = Locate(String1, TRUE);
  } // end while

  finish:
  Refresh();
  return OK;
} // end ExecuteChange


void FileWindows::SplitLine()
{
  if (theCursor.CurLineOf()->isReadOnly)
    return;
  if (theFile.InsertLine(&theCursor) != OK) // Insert a new line
    return;
  Lines *pNext = theCursor.CurLineOf()->NextOf();
  pNext->SetData(theCursor.EOL());
  EraseEOL();			       // Erase to end of line
  theEditBuffer.IncrChanges();
  Refresh();
} // end SplitLine


void FileWindows::JoinLines()
{
  Lines* pCurLine = theCursor.CurLineOf();
  Lines* pNext    = pCurLine->NextOf();

  if (pCurLine->isReadOnly || pNext->isReadOnly)
    return;

  int NewLength = strlen(pCurLine->DataOf()) - 1;

  while (pCurLine->DataOf()[NewLength] == ' ')
    --NewLength;

  NewLength += 2;

  pCurLine->ReplaceString(NewLength, 0, pNext->DataOf());
  Cursors aCursor;
  aCursor = theCursor;
  aCursor.Move(DOWN);
  theFile.DeleteLine(&aCursor);
  theEditBuffer.IncrChanges();
  Refresh();
} // end JoinLines


static char* StrStr(char* Str1, char* Str2, bool isCaseSensitive)
{
  int (*pStrncmp)(const char *, const char *, size_t);

  int Str2Len = strlen(Str2);
  if (isCaseSensitive)
    pStrncmp = strncmp;
  else
    pStrncmp = strnicmp;

  while (*Str1)
  {
    if ((*pStrncmp)(Str1, Str2, Str2Len) == 0)
      return Str1;
    ++Str1;
  } // endwhile

  return NULL;
} // end StrStr