//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : fwindow.h
//
// Classes     : FileWindows
//
// Description : derived from InputWindows, displays the file which it         
//               contains at the current position.
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#if !defined(FWINDOW_H)
#define FWINDOW_H

#include "Globals.h"
#include "Iwindow.h"
#include "File.h"

class Marks;

class FileWindows : public InputWindows
{
  friend Marks;

public :

  FileWindows(char *);                 // Load a file into the window.

  void Top();                          // Top of file.

  void Bottom();                       // Bottom of file.

  void Goto(int);                      // Go to line number.

  void PgDn();                         // Page down over file.

  void PgUp();                         // Page up over file.

  void Enter();                        // CR or insert line.

  void SplitLine();                    // Split a line.

  void JoinLines();                    // Join two lines.

  void Save()                          // Save the file to disk.
    { theFile.Save(); theEditBuffer.resetChanges(); }

  void DeleteLine()
    { theFile.DeleteLine(&theCursor); Refresh(); }

  char *FileNameOf()
    { return theFile.NameOf(); }

  void Rename(char*);

  int CursorColOf()
    { return theCursor.CurColOf() + 1; }

  int CursorRowOf()
    { return theCursor.CurRowOf(); }

  void MarkLine(Marks&);

  FileWindows* PrevOf()
    { return thePrev; }

  FileWindows* NextOf()
    { return theNext; }

  void setPrev(FileWindows* aFileWindow)
    { thePrev = aFileWindow; }

  void setNext(FileWindows* aFileWindow)
    { theNext = aFileWindow; }

  RETCODE Find(char*, bool = FALSE);

  RETCODE Change(char*);

  void ResetCursor();                 // Put the cursor data back.

// For base class InputWindows

  RETCODE Refresh();                   // Refresh the display of the window.

private:

// Private variables and functions

  RETCODE ExecuteChange(char*, char*, CHANGE_OPTS);

  RETCODE Locate(char*, bool);

  void  RowsUp(int);

  Files theFile;                       // The file in the window.

  FileWindows *theNext,
	      *thePrev;

// Find/change parameters

  bool isCaseSensitive;
  bool isFirstLocate;

  Lines FirstParm;

}; // end class FileWindows

#endif
