//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : iwindow.cpp
//
// Classes     : InputWindows
//
// Description : derived from Windows, base class for all windows needing
//               keyboard input, i.e. file and command windows.         
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#include <conio.h>
#include <string.h>

#include "Iwindow.h"
#include "Keyboard.h"


RETCODE InputWindows::CursorUp()
{
  RETCODE rc = OK;
  if (isFarUp(theCursor))
    rc = Scroll(UP);
  else
    theCursor.Move(UP);                // Move cursor up.
  return rc;
} // end CursorUp


RETCODE InputWindows::CursorDown()
{
  RETCODE rc = OK;
  if (isFarDown(theCursor))
    rc = Scroll(DOWN);
  else
    theCursor.Move(DOWN);              // Move cursor down.
  return rc;
} // end CursorDown


RETCODE InputWindows::CursorLeft()
{
  RETCODE rc = OK;                     // Is the cursor at the left
  if (isFarLeft(theCursor))            // of the window?
    rc = Scroll(LEFT);
  else
    theCursor.Move(LEFT);
  return rc;
} // end CursorLeft


RETCODE InputWindows::CursorRight()
{
  RETCODE rc = OK;
  if (isFarRight(theCursor))
    rc = Scroll(RIGHT);
  else
    theCursor.Move(RIGHT);
  return rc;
} // end InputWindows::CursorRight


void InputWindows::CursorHome()
{
  theCursor.Home();
  if (theBaseCol > 1)
  {
    theBaseCol = 1;
    Refresh();
  }
  else
    theCursor.Show();
} // end CursorHome


void InputWindows::Tab()
{
  MoveCursorColTo(theCursor.CurColOf() + 8);
}// end Tab


void InputWindows::BackTab()
{
  int NewPos = theCursor.CurColOf() - 8;
  if (NewPos < 0)
    NewPos = 0;
  MoveCursorColTo(NewPos);
} // end BackTab


void InputWindows::MoveCursorColTo(int aDataOffset)
{
  if (aDataOffset == theCursor.CurColOf())  // Already there?
    return;

  if (aDataOffset == 0)
  {
    CursorHome();
    return;
  } // endif

  if (aDataOffset + 2 - theBaseCol > theNoOfCols  // screen too far left
   || theBaseCol - 1 > aDataOffset)               // screen too far right
  {
    theBaseCol = aDataOffset + 2 - theNoOfCols;
    if (theBaseCol < 1)
      theBaseCol = 1;
    theCursor.setCurCol(aDataOffset);
    theCursor.setCol(aDataOffset + 2 - theBaseCol);
    Refresh();
  }
  else
  {
    theCursor.setCurCol(aDataOffset);
    theCursor.setCol(aDataOffset + 2 - theBaseCol);
    theCursor.Show();
  } // endif

} // end MoveCursorColTo


void InputWindows::CursorEnd()
{
  char* Data = theCursor.CurLineOf()->DataOf();

  int offset = strlen(Data);           // Find end of data.
  while (--offset >= 0)
    if (Data[offset] != ' ')
      break;

  MoveCursorColTo(++offset);           // We want cursor at column after
				       // end of data.
} // end CursorEnd


void InputWindows::Type(char aChar)
{
  RETCODE rc;
  if (theKeyboard.isInInsertMode())    // Insert the character.
    rc = theEditBuffer.InsertChar(aChar);
  else                                 // Replace a character.
    rc = theEditBuffer.ReplaceChar(aChar);
  if (rc ==OK)
    CursorRight();                     // Move cursor to next character.
  RefreshLine();
} // End Type


void InputWindows::BackSpace()
{
  if (CursorLeft() == OK)
    theEditBuffer.DeleteChar();
  RefreshLine();
} // End Backspace


void InputWindows::DeleteChar()
{
  theEditBuffer.DeleteChar();
  RefreshLine();
} // End DeleteChar


void InputWindows::Rollback()
{
  theEditBuffer.Rollback();
  RefreshLine();
} // end Rollback

          
void InputWindows::EraseEOL()
{
  theEditBuffer.EraseEOL();  
  RefreshLine();
} // End DeleteChar


void InputWindows::RefreshLine()
{
  DisplayLine(theCursor.CurLineOf(), theCursor.RowOf(), theBaseCol);
  theCursor.Show();
} // end InputWindows::RefreshLine


RETCODE InputWindows::Scroll(direction aDir)
{
  switch (aDir)
  {
    case RIGHT :
      ++theBaseCol;
      theCursor.setCurCol(theCursor.CurColOf() + 1);
      Refresh();
      break;

    case LEFT :
      if (theBaseCol == 1)
	return CANT_MOVE;
      --theBaseCol;
      theCursor.setCurCol(theCursor.CurColOf() - 1);
      Refresh();
      break;

    case DOWN :
      if (theLastRow->NextOf() == NULL)
	return CANT_MOVE;
      theBaseRow = theBaseRow->NextOf();
      theLastRow = theLastRow->NextOf();

      Windows::Scroll(DOWN, 1);
      DisplayLine(theLastRow, theNoOfRows, theBaseCol);
      theCursor.setCurRow(theCursor.CurRowOf() + 1);
      theCursor.setCurLine(theCursor.CurLineOf()->NextOf());
      theCursor.Show();
      break;

    case UP :
      if (theBaseRow->PrevOf() == NULL)
	return CANT_MOVE;
      theBaseRow = theBaseRow->PrevOf();
      theLastRow = theLastRow->PrevOf();

      Windows::Scroll(UP, 1);
      DisplayLine(theBaseRow, 1, theBaseCol);
      theCursor.setCurRow(theCursor.CurRowOf() - 1);
      theCursor.setCurLine(theCursor.CurLineOf()->PrevOf());
      theCursor.Show();
      break;

  } // endswitch

  return OK;
} // end InputWindows::Scroll
