//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : iwindow.h  
//
// Classes     : InputWindows
//
// Description : derived from Windows, base class for all windows needing
//               keyboard input, i.e. file and command windows.         
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#if !defined(IWINDOW_H)
#define IWINDOW_H

#include "Globals.h"
#include "Window.h"
#include "Ebuf.h"

class InputWindows : public Windows
{
public :

  InputWindows()
    { theEditBuffer.SetCursor(&theCursor); }

  void Type(char);

  void Rollback();

  RETCODE CursorLeft();

  RETCODE CursorRight();

  RETCODE CursorDown();

  RETCODE CursorUp();

  void Tab();

  void BackTab();

  void CursorHome();

  void CursorEnd();

  void DeleteChar();

  void BackSpace();

  void EraseEOL();

  int NoOfChanges()
    { return theEditBuffer.NoOfChanges(); }

  // Data is needed for the pure virtual functions, so they are
  // defined in the derived classes (File and Command windows).

  virtual void Enter() = 0;

  virtual RETCODE Refresh() = 0;       // Refresh the display of the window.

protected :

  RETCODE Scroll(direction);

  void RefreshLine();

  void MoveCursorColTo(int);           // Move cursor to DATA column.

  Cursors     theCursor;               // Every input window has a cursor!
  EditBuffers theEditBuffer;           // A line editing buffer.

  int   theBaseCol;                    // First col. displayed in window.
  Lines *theBaseRow,                   // First line displayed in window.
	*theLastRow;                   // Last line displayed in window.

}; // end class InputWindows

#endif
