//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : keyboard.h
//
// Classes     : Keyboards
//
// Description : interface to keyboard, including keystroke to editor
//               function mapping.     
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#include <conio.h>
#include <ctype.h>

#include "keyboard.h"

const int NO_OF_CODES = 256;


static OPCODE key_to_opcode[NO_OF_CODES] = {
 NOP, NOP, NOP, NOP, NOP, ERASE_EOL, FIND_AGAIN, NOP, // 0 - 7
  BACKSP, TAB, NOP, NOP, NOP, ENTER, NOP, NOP,   // 8 - 15
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,      // 16 - 23
  NOP, NOP, NOP, COMMAND, NOP, NOP, NOP, NOP, // 24 - 31
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,      // 32 - 39
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,     // 40 - 47
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,      // 48 - 55
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,     // 56 - 63
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,      // 64 - 71
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,     // 72 - 79
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,      // 80 - 87
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,     // 88 - 95
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,      // 96 - 103
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,     // 104 - 111
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,      // 112 - 119
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, DELETE_LINE, // 120 - 127
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,      // 128 - 135
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP
};


static OPCODE extended_to_opcode[NO_OF_CODES] = {
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,                 // 0 - 7
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, BACK_TAB,           // 8 - 15
 NOP, NOP, NOP, NOP, NOP, NOP, UNMARK, NOP,              // 16 - 23
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, SPLIT,              // 24 - 31
 DELETE_MARK, NOP, NOP, NOP, JOIN, NOP, MARK_LINE, NOP,  // 32 - 39
  NOP, NOP, NOP, NOP, NOP, NOP, COPY_MARK, NOP,          // 40 - 47
 NOP, NOP, MOVE_MARK, NOP, NOP, NOP, NOP, NOP,           // 48 - 55
  NOP, NOP, NOP, HELP, SAVE, QUIT, SAVEQUIT, DOS,        // 56 - 63
 NOP, COMMAND_NAME, COMMAND_EDIT, UNDO, NEXT_FILE, NOP, NOP, HOME, // 64 - 71
  CUR_UP, PGUP, NOP, CUR_LEFT, NOP, CUR_RIGHT, NOP, END, // 72 - 79
 CUR_DOWN, PGDN, INSERT, DELETE, NOP, NOP, NOP, NOP,     // 80 - 87
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,                // 88 - 95
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,                 // 96 - 103
  NOP, NOP, NOP, NOP, NOP, NOP, SHIFT_MARK_LEFT, SHIFT_MARK_RIGHT, // 104 - 111
 NOP, PREV_FILE, NOP, NOP, NOP, BOTTOM_OF_FILE, NOP, TOP_OF_FILE,   // 112 - 119
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
 NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP,
  NOP, NOP, NOP, NOP, NOP, NOP, NOP, NOP
};


OPCODE Keyboards::getOp()
{
  if (theQueuedOp != NOP)
  {
    OPCODE tempOp = theQueuedOp;
    theQueuedOp = NOP;
    return tempOp;
  } // endif

  const int EXTENDED = 0;
  int ch = getch();

  if (theQueuedParm.DataOf()[0] != '\0')
    theQueuedParm.SetData("");

  OPCODE OpCode;

  if (ch == EXTENDED)
  {
    ch = getch();
    OpCode = extended_to_opcode[ch];
  }
  else
  {
    if (key_to_opcode[ch] == NOP && isprint(ch))
    {
      if (isprint(ch))
      {
	ascii_char = ch;
	OpCode = ASCII;
      }
      else
	OpCode = NOP;
    }
    else
      OpCode = key_to_opcode[ch];
  } // endif

  return OpCode;
} // end getOp
