//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : line.h
//
// Classes     : Lines
//
// Description : this class holds a line of data for InputWindows.         
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#include <mem.h>
#include <string.h>
#include <stdtempl.h>                  // For max() template.

#include "line.h"


Lines::Lines(char* aString)
{
  theLength = 0;
  theData = NULL;
  thePrev = theNext = NULL;
  isReadOnly = isMarked = FALSE;
  if (aString != NULL)
    SetData(aString);
} // end Lines constructor


RETCODE Lines::SetData(char *aString)
{
  int NewLength = strlen(aString);

  if (theData != NULL)
    delete theData;

  if (aString[NewLength - 1] == '\n')
    aString[--NewLength] = '\0';

  if ((theData = new char[NewLength + 1]) == NULL)
    return NO_MORE_MEMORY;

  theLength = NewLength;
  strcpy(theData, aString);
  return OK;
} // end Lines::SetData


void Lines::Replace(Lines *aLine)      // Swap contents of lines.
{
  char *tempData;
  int  tempLength;

  tempData = theData;
  theData  = aLine->theData;
  aLine->theData = tempData;

  tempLength = theLength;
  theLength  = aLine->theLength;
  aLine->theLength = tempLength;
} // end Cursors::replace_line


char *Lines::DataOf(int aCol)
{
  if (aCol > theLength)
    aCol = theLength + 1;

  return &theData[aCol - 1];
} // end Lines::DataOf


void Lines::EraseEOL(int Offset)
{
  char *pCh = &theData[Offset];
  while (*pCh)
    *pCh++ = ' ';
} // end EraseEOL


void Lines::ReplaceChar(char aChar, int Offset)
{
  if (Offset > theLength - 1)
    Lengthen(Offset);
  theData[Offset] = aChar;
} // end Lines::ReplaceChar


void Lines::InsertChar(char aChar, int Offset)
{
  if (Offset > theLength - 1 || theData[theLength - 1] != ' ')
    Lengthen(Offset);

  memmove(&theData[Offset + 1],
	  &theData[Offset], theLength - Offset - 1);
  theData[Offset] = aChar;
} // end Lines::InsertChar


static bool isNotBlank(char* pCh)
{
  while (*pCh)
   if (*pCh++ != ' ')
     return TRUE;
  return FALSE;
} // end isNotBlank


void Lines::ReplaceString(int anOffset, int Str1Len, char* String2)
{
  // Replace string length Str1Len at position anOffset with String2.

  for (int OrgLen = theLength; theData[OrgLen - 1] == ' '; --OrgLen)
    ;

  int Str2Len = strlen(String2);       // Length of string to replace with.
  int Extra   = Str2Len - Str1Len;

  if (Str1Len < Str2Len)
  {                                    // Line will be longer...
    if (anOffset > theLength - Extra - 1)
      Lengthen(anOffset + Extra);
    else if (isNotBlank(&theData[theLength - Extra - 1]))
      Lengthen(theLength + Extra);
  } // end if

  if (Str2Len != Str1Len)
  {                                    // Move data for replacement.
    if (OrgLen - anOffset - Str1Len > 0)
      memmove(&theData[anOffset + Str2Len],
	      &theData[anOffset + Str1Len],
	      OrgLen - anOffset - Str1Len);
    for (int off = OrgLen + Extra; theData[off] != '\0'; ++off)
      if (theData[off] != ' ')
	theData[off] = ' ';
  } // end if

  memcpy(&theData[anOffset], String2, Str2Len); // Copy the data.

} // end ReplaceString


RETCODE Lines::Lengthen(int anOffset)
{
  int NewLength = max(theLength, anOffset + 1) + 10;
  char *NewData = new char[NewLength + 1];

  if (NewData == NULL)
    return NO_MORE_MEMORY;

  strcpy(NewData, theData);
  delete theData;
  theData = NewData;
  memset(&theData[theLength], ' ', NewLength - theLength);
  theLength = NewLength;
  theData[theLength] = '\0';

  return OK;
} // end Lines::Lengthen


void Lines::Strip()
{
  int NewLength = theLength - 1;

  while (theData[NewLength] == ' ')
    --NewLength;

  if (++NewLength < theLength)
    theData[NewLength] = '\0';
  else
    return;
  char *NewData = new char[NewLength + 1];
  strcpy(NewData, theData);
  delete theData;
  theData = NewData;
  theLength = NewLength;
} // end Lines::Strip


void Lines::DeleteChar(int anOffset)
{
  if (anOffset > theLength - 1)
    return;
  if (theLength - anOffset - 1 > 0)
    memmove(&theData[anOffset],
	    &theData[anOffset + 1], theLength - anOffset - 1);
  theData[theLength - 1] = ' ';
} // end Lines::DeleteChar
