//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : mark.h
//
// Classes     : Marks
//
// Description : this class holds the information required for the 'marked'
//               lines available for copying/moving/deleting.  These lines
//               can be moved between files, so there is one mark per
//               editor, not per file.     
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#if !defined(MARK_H)
#define MARK_H

#include "Globals.h"

#include "Fwindow.h"

class Marks
{
public:

  Marks()                              // Constructor.
   { Reset(); }

  void Set(Cursors&, FileWindows&);    // Set a marked area.

  void Clear();                        // Clear the mark i.e. unmark.

  void Delete(bool = TRUE);            // Delete marked block of lines.

  void Copy(FileWindows&, bool = TRUE);// Copy marked block.

  void Move(FileWindows&);             // Move marked block.

  void Shift(direction);               // Shift marked block right or left.

  FileWindows *FileWindowOf()          // Which file is marked?
    { return theMarkedFileW; }

private:

  void   Reset();                      // Init./reset class state.

  FileWindows *theMarkedFileW;         // The file in which the mark is.

  int    theNoOfLines;                 // The no. of lines in marked block.

  Lines  *theStartLine,                // The first marked line.
	 *theEndLine;                  // The last marked line.

  int    theStartRow,                  // Line no. of first marked line.
	 theEndRow;                    // Line no. of last marked line.

  int    theStartCol,                  // Start and end columns of mark
	 theEndCol;                    // for block marks - not yet used.

}; // end class Marks

#endif
