//////////////////////////////////////////////////////////////////////////////
//
// Project     : a DOS full-screen editor using the object-oriented
//               facilities of C++.
//
// File name   : window.cpp         
//
// Classes     : Windows
//
// Description : the base class for all screen windows, including file,
//               command and message windows.     
//
// Copyright   : (c) 1991, 1993 Ian Craggs
//
//////////////////////////////////////////////////////////////////////////////
          
#include <conio.h>
#include <_video.h>
#include <string.h>

#undef UP
#undef DOWN

#include "Window.h"
#include "Util.h"


Windows* Windows::ActiveWindow = NULL; // No active window to start.

void Windows::Setup(int aLeft, int aTop, int aRight, int aBottom,
		    int aColor, int aBackColor)
{
  theNoOfRows = aBottom - aTop + 1;
  theNoOfCols = aRight  - aLeft + 1;
  theLeft     = aLeft;
  theTop      = aTop;
  theRight    = aRight;
  theBottom   = aBottom;
  theColor    = aColor;
  theBackColor= aBackColor;
} // end Windows::Setup


void Windows::Reverse()
{
  int Temp = theColor;
  theColor = theBackColor;
  theBackColor = Temp;
  textcolor(theColor);
  textbackground(theBackColor);
} // end Windows::Reverse


void Windows::Activate()
{
  if (ActiveWindow != this)
  {
    ActiveWindow = this;
    window(theLeft, theTop, theRight, theBottom);
    textcolor(theColor);
    textbackground(theBackColor);
  } // endif
} // end Windows::Activate


void Windows::DisplayLine(char *aString, int aRowNo)
{
  Activate();
  gotoxy(1, aRowNo);

  int StrLen = strlen(aString);

  if (StrLen < theNoOfCols)
  {
    PutString(aString, StrLen);
    clreol();
  }
  else
    PutString(aString, theNoOfCols);

} // end Windows::DisplayLine


void Windows::DisplayLine(Lines *aLine, int aRowNo, int aColNo)
{
  Activate();
  gotoxy(1, aRowNo);

  char *aString = aLine->DataOf(aColNo);

  int StrLen = strlen(aString);

  if (aLine->isMarked)
    Reverse();

  if (StrLen < theNoOfCols)
  {
    PutString(aString, StrLen);
    clreol();
  }
  else
    PutString(aString, theNoOfCols);

  if (aLine->isMarked)
    Reverse();
} // end DisplayLine


void Windows::Scroll(direction dir, int noOfLines)
{
  if (dir == DOWN)
    movetext(theLeft, theTop + noOfLines, theRight,
	     theBottom, theLeft, theTop);
  else
    movetext(theLeft, theTop, theRight,
	     theBottom - noOfLines, theLeft, theTop + noOfLines);
} // end Scroll
